/* Sistemas Integrados TECNISA
   Planejamento Alimentao Industrial

Nome: 7105_PLASTPO.SQL (CONSPLAN_GERAL BACKGROUND)

Descrio:
Clculo do Consumo Planejado Consolidado
(Cardpio, Consumo por Servio Fixo ou
Percapita) e gravao na tabela CONSPLAN.

P_NRLISTA      - Nmero da Lista de Servios
P_DTINICIAL    - Data inicial
P_DTFINAL      - Data final
P_IDPRODUTOS   - Identificador dos produtos a serem gravados.
                 Formato "XXX"
                 1 posio = "C" -> Cardpio
                 2 posio = "F" -> Consumo por Servio Fixo
                 3 posio = "P" -> Consumo por Servio Percapita
P_IDSEPPRATO   - Identificador de requisio analtica, ou seja, separada
                 por prato ou por setor, consumo fixo e consumo percapita.
                 Formato "S" (setor), "P" (prato) ou "N" (n/a).
P_PEACRESCIMO  - Percentual de acrscimo
P_IDARREDOND   - Identificador de utilizao do arredondamento
                 para requisio do Produto. Formato
                    S - ARREDONDAMENTO DE REQUISIO
                    N - NAO ARREDONDAR
                    C - ARREDONDAMENTO DE COMPRAS
P_CDPRODUTOINI - Cdigo do Produto inicial
P_CDPRODUTOFIN - Cdigo do Produto final
P_CDOPERADOR   - Cdigo do Operador
P_NRCONSPLANTH - Nuremo do proximo codigo, quando acionado segundo plano BACKGROUND 
P_NRCONSPLAN   _ NOVOCODIGO no formato 'P000000001'. (CDCONTADOR = CONSPLANPREV, nico para todas as filiais)

*/

IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
                    WHERE
	        O.NAME = 'CONSPLAN_GERAL'  AND
	        TYPE = 'P' 		          AND
	        U.UID = O.UID                     AND
	        U.NAME = USER)
  DROP PROCEDURE CONSPLAN_GERAL
  GO

CREATE PROCEDURE CONSPLAN_GERAL
   @P_NRLISTA       VARCHAR(10),
   @P_DTINICIAL     DATETIME,
   @P_DTFINAL       DATETIME,
   @P_IDPRODUTOS    VARCHAR(3),
   @P_IDSEPPRATO    VARCHAR(1),
   @P_PEACRESCIMO   DOUBLE PRECISION,
   @P_IDARREDOND    VARCHAR(1),
   @P_CDPRODUTOINI  VARCHAR(20),
   @P_CDPRODUTOFIN  VARCHAR(20),
   @P_CDOPERADOR    VARCHAR(12),
   @P_NRCONSPLANTH  VARCHAR(10),
   @P_NRCONSPLAN    VARCHAR(10) OUTPUT
AS
   DECLARE @V_CDARVINI    VARCHAR(20);
   DECLARE @V_CDARVFIN    VARCHAR(20);

   DECLARE @V_SERVICOS    VARCHAR(4000);
   DECLARE @V_PRXCONSPLAN VARCHAR(20);
   DECLARE @V_NRCONSPLAN  VARCHAR(10);

   DECLARE @CUR_FILIAL_CDFILIAL VARCHAR(4);
   DECLARE @CUR_SERVICO_CDSERVFILI VARCHAR(5);
   DECLARE @V_QTCARACTER INT;
BEGIN
   SET @V_CDARVINI = @P_CDPRODUTOINI;
   
   SELECT @V_QTCARACTER = LEN(CDPICTPROD) - (SELECT MAX(CDNVPRODUTO) - 1 FROM PRODUTO)
   FROM   PARAMGERAL
   WHERE  CDPARGERAL = '0001';
   SET @V_CDARVFIN = LEFT(@P_CDPRODUTOFIN + REPLICATE('z', @V_QTCARACTER), @V_QTCARACTER);

   IF (@P_NRCONSPLANTH = 'GERANOVCOD') BEGIN
      SET @V_NRCONSPLAN = '_';
      EXEC NOVO_CODIGO 'CONSPLANPREV', @P_SEQUENCIAL = @V_PRXCONSPLAN OUTPUT, @P_QTDE = 1;
      SET @V_NRCONSPLAN = 'P'+SUBSTRING(@V_PRXCONSPLAN,12,9);
   END
   ELSE BEGIN
      SET @V_NRCONSPLAN = @P_NRCONSPLANTH;
   END;
   
   DECLARE CUR_FILIAL CURSOR FOR
       SELECT
          DISTINCT SUBSTRING(LISTA.CDLISTA,1,4) AS CDFILIAL
       FROM
          LISTA
       WHERE
          (LISTA.NRLISTA = @P_NRLISTA)
   OPEN CUR_FILIAL;
   FETCH NEXT FROM CUR_FILIAL INTO @CUR_FILIAL_CDFILIAL;
   WHILE (@@FETCH_STATUS = 0)
   BEGIN
      SET @V_SERVICOS = '_';

      DECLARE CUR_SERVICO CURSOR FOR
          SELECT
             SUBSTRING(CDLISTA, 5, 5) AS CDSERVFILI
          FROM
             LISTA
          WHERE
             NRLISTA = @P_NRLISTA AND
             SUBSTRING(CDLISTA, 1, 4) = @CUR_FILIAL_CDFILIAL;

      OPEN CUR_SERVICO;
      FETCH NEXT FROM CUR_SERVICO INTO @CUR_SERVICO_CDSERVFILI;
      WHILE (@@FETCH_STATUS = 0)
      BEGIN
         SET @V_SERVICOS = @V_SERVICOS + @CUR_SERVICO_CDSERVFILI + '_';
         FETCH NEXT FROM CUR_SERVICO INTO @CUR_SERVICO_CDSERVFILI;
      END;
      CLOSE CUR_SERVICO;
      DEALLOCATE CUR_SERVICO;

      EXEC CONSUMO_PLANEJADO_PLA @CUR_FILIAL_CDFILIAL,
                            @V_SERVICOS,
                            @P_DTINICIAL,
                            @P_DTFINAL,
                            'C',   /* GRAVAO NA TABELA CONSPLAN */
                            @P_IDPRODUTOS, /* CARDPIO, CONSUMO FIXO E PERCAPITA */
                            @P_IDSEPPRATO,   /* FOI INCLUDO O PARMETRO P_IDSEPPRATO */
                            @P_PEACRESCIMO,
                            @P_IDARREDOND,
                            @V_NRCONSPLAN,
                            @V_CDARVINI,
                            @V_CDARVFIN,
                            @P_CDOPERADOR,
                            NULL,
                            NULL;

      FETCH NEXT FROM CUR_FILIAL INTO @CUR_FILIAL_CDFILIAL;
   END;
   CLOSE  CUR_FILIAL;
   DEALLOCATE CUR_FILIAL;
   SET @P_NRCONSPLAN = @V_NRCONSPLAN;
   DELETE FROM LISTA WHERE NRLISTA = @P_NRLISTA;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'CONSPLAN_GERAL','5.03.006';
GO
