import Vue from 'vue';
import Router from 'vue-router';
import { Config } from '@zeedhi/core';
import { TekLibLocalStorage, TekLibAuth } from '@zeedhi/tek-lib';
import routes from './routes';
import Store from '../store';

Vue.use(Router);

const router = new Router({
  mode: 'history',
  base: Config.env.BASE_URL,
  routes,
});

router.beforeEach(async (to, from, next) => {
  if (window.parent !== window && to.name !== 'token-error' && to.query.isZhAngular === '1') { // if application is called inside a iframe
    if (to.query.token) {
      localStorage.setItem('LOGIN_TOKEN', to.query.token.toString());
      localStorage.setItem('LOGIN_USER', to.query.user.toString());
      localStorage.setItem('LOGIN_LANGUAGE', to.query.languange.toString());
      localStorage.setItem('PRODUCT_ID', to.query.productId.toString());
      if (to.query.hash !== '0') {
        localStorage.setItem('LOGIN_HASH', to.query.hash.toString());
      }
      Store.commit('setAppLoginToken', to.query.token);
      Store.commit('setAppDisplayOptions', { menu: to.query.menu, header: to.query.header });
    }

    if (!Store.state.appLoginToken) {
      Store.commit('setAppDisplayOptions', { menu: '0', header: '0' });
      next('/token-error');
      return;
    }

    delete to.query.menu;
    delete to.query.header;
    delete to.query.token;

    TekLibLocalStorage.setItem('IS_NEXT', false);
    TekLibLocalStorage.setItem('LOGIN_KEEP_CONNECTED', 'Yes');
    TekLibLocalStorage.removeItem('routeParams');

    next();
  } else {
    TekLibLocalStorage.setItem('IS_NEXT', true);
  }

  TekLibAuth.setInitialData(to.path, Config.env.loginRoutePath);
  if (to.path !== Config.env.loginRoutePath && !localStorage.getItem('LOGIN_TOKEN')) {
    next(Config.env.loginRoutePath);
  } else if (to.path === Config.env.loginRoutePath && localStorage.getItem('LOGIN_TOKEN')) {
    next(Config.env.baseUrl);
  } else {
    next();
  }
});

router.afterEach((to, from) => {
  const htmlElement = document.getElementsByTagName('html')[0];
  if (to.name === 'microservice') {
    htmlElement.setAttribute('microservice', '');
    Store.commit('setRouterParams', to.params);
    const frame = (document.getElementById('microservice-frame') as HTMLIFrameElement);
    if (frame && frame.contentWindow && to.path.split('/')[2] === from.path.split('/')[2]) {
      frame.contentWindow.postMessage({
        type: 'microservice-route-change',
        routerParams: to.params,
      }, `${Config.env.modules[to.path.split('/')[2]].baseUrl}`);
    }
    window.document.title = `${Config.env.product.name} - ${to.params.page}`;
  } else {
    htmlElement.removeAttribute('microservice');
    window.document.title = Config.env.product.name;
  }

  if (to.name === 'login') {
    htmlElement.setAttribute('login', '');
  } else {
    htmlElement.removeAttribute('login');
  }
});

export default router;
